"use strict";

const postcss = require('gulp-postcss');
const autoprefixer = require("autoprefixer");
const Task = require("../task");
const browserslist = require("../../../../package.json").browserslist;
const header = require("gulp-header");

class ProcessScss extends Task {
    constructor(name, taskDepend, taskDependencies, gulp, helper, conf, project, watchMode, watchTasks) {
        super(name, taskDepend, taskDependencies, gulp, helper, conf, project);
        this.watchMode = watchMode;
        this.watchTasks = watchTasks;
    }

    task(gulp, helper, conf, project) {
        return (done) => {
            if (this.watchMode) {
                gulp.watch(conf.sassConfiguration.sass.inputFilter, this.watchTasks)
                    .on("change", (event) => {
                        helper.debug("Fichier ", event.path, "a été", event.type);
                    });
                done();
            } else {
                this.sassProcess(gulp, helper, conf, project,  done);
            }
        }
    }

    /**
     * Méthode de génération des fichiers css depuis des scss
     */
    sassProcess(gulp, helper, conf, project, done) {
        //inputFilters
        if (!Array.isArray(conf.sassConfiguration.sass.inputFilter)) {
            conf.sassConfiguration.sass.inputFilter = [conf.sassConfiguration.sass.inputFilter];
        }
        conf.sassConfiguration.sass.inputFilter.splice(0, -1, conf.sassConfiguration.img.output.dir + "_sass-image.scss");

        let sass;

        try {
            sass = require("gulp-sass");
        } catch (e) {
            helper.error(e);
            done();
        }

        // Génération des variables sass pour le path des ressources
        let data = '$PATH_FONT: "' + project.staticPath + 'theme/fonts/"; $PATH_IMG: "' + project.staticPath + 'theme/img/";';

        // Injection des variables via header
        let sassPipe = gulp.src(conf.sassConfiguration.sass.inputFilter)
            .pipe(header(data))
            .pipe(sass(conf.sassConfiguration.sass.options).on('error', sass.logError));

        // output
        return helper.stream(
            done,
            sassPipe
                .pipe(postcss([autoprefixer({ browsers: browserslist })]))
                .pipe(gulp.dest(conf.sassConfiguration.sass.output.dir))
        );
    }
}

module.exports = ProcessScss;
