"use strict";

const State = require("../../state");
const Utils = require("../utils");

const Task = require("../task");
const commander = require("../../../gulp/commander");

class FixVersion extends Task {
    constructor(name, taskDepend, taskDependencies, gulp, helper, conf, project) {
        super(name, taskDepend, taskDependencies, gulp, helper, conf, project);

        if (!State.version) {
            let timestamp = new Date();
            if (helper.getVersion()) {
                State.version = helper.getVersion() != "auto" ? helper.getVersion() : ("-" + timestamp.getFullYear() + (timestamp.getMonth() + 1) + timestamp.getDay() + timestamp.getHours() + timestamp.getMinutes() + timestamp.getSeconds());
                helper.info("State.version " + State.version);
            }
        }
    }

    task(gulp, helper, conf, project) {
        return (done) => {

            if(helper.getVersion().toUpperCase() === "RC") {

                let args = ["view"];
                if (helper.getPublishRegistry()) {
                    args = args.concat(["--registry", helper.getPublishRegistry()]);
                }
                args.push("--json");
                args.push(project.name + "@'~" + project.version + "-RC'");
                args.push("version");

                return commander.toPromise({ cmd: "npm", args: args, cwd: helper.getMainProcessDir() }, true).then((data) => {
                    let newVersion = "-RC1";
                    let oldLastVersion = 0;
                    if(data) {
                        let npmReturn = JSON.parse(data);
                        if(!npmReturn.error) {
                            if(!Array.isArray(npmReturn)) {
                                npmReturn = [npmReturn];
                            }
                            npmReturn.forEach((ver) => {
                                let variables = ver.match(/\-RC(\w+)$/);
                                if (variables && variables.length > 0 && oldLastVersion < variables[1]) {
                                    oldLastVersion = variables[1];
                                }
                            });
                            newVersion = "-RC" + ( 1 + Number.parseInt(oldLastVersion));
                        } else {
                            helper.warn(" NPM VIEW error: " + npmReturn.error.summary);
                        }
                    }
                    State.version = newVersion;
                    
                    this.replaceInModules(project, helper.DEPENDENCIES, helper, project.packageJson.version);
                    this.replaceInModules(project, helper.DEV_DEPENDENCIES, helper, project.packageJson.version);
                    
                    project.packageJson.version = this.getVersion(project);
                    project.version = project.packageJson.version;
                    
                    helper.info("Version fixée: " + project.packageJson.name, "@" , project.packageJson.version);

                    return helper.stream(
                        done,
                        gulp.src(["package.json"])
                            .pipe(Utils.packageJsonFormatter(helper, project))
                            .pipe(gulp.dest("."))
                    );


                }).catch((err) => {
                    helper.error(`La commande npm ${args} dans ${helper.getMainProcessDir()} est ko`);
                    done(err);
                });
            } else {

                if(!State.version) {
                    helper.error("Erreur pas de version fix précisée, utiliser l'argument '--versionFix' !!");
                    return done();
                }
                
                
                this.replaceInModules(project, helper.DEPENDENCIES, helper);
                this.replaceInModules(project, helper.DEV_DEPENDENCIES, helper);
                
                project.packageJson.version = this.getVersion(project);
                project.version = project.packageJson.version;
                
                helper.info("Version fixée: " + project.packageJson.name, "@" , project.packageJson.version);

                return helper.stream(
                    done,
                    gulp.src(["package.json"])
                        .pipe(Utils.packageJsonFormatter(helper, project))
                        .pipe(gulp.dest("."))
                );
            }

        }
    }

    replaceInModules(project, KeyDependencies, helper, versionToFix) {
        if (State.moduleList && project.packageJson[KeyDependencies]) {
            Object.keys(State.moduleList).forEach(projectName =>  {
                if (project.packageJson[KeyDependencies][projectName]) {
                    let version = this.getVersion(State.moduleList[projectName]);

                    helper.debug("ReplaceInModules " + project.packageJson.name, "@",project.packageJson.version,
                        ", KeyDependencies : ",KeyDependencies, ", projectName : ", projectName, ", version : ", project.packageJson[KeyDependencies][projectName], "=>", version);

                    project.packageJson[KeyDependencies][projectName] = this.getVersion(State.moduleList[projectName]);
                }
            });
        }
    }

    getVersion(currentProject) {
        if (currentProject.versionFix) {
            return currentProject.packageJson.version;
        } else if (State.version.match(/^\-|\\./) ) {
            return currentProject.packageJson.version + State.version;
        } else {
            return State.version;
        }
    }
}


module.exports = FixVersion;